@@ -11,7 +11,9 @@ import logging
 import traceback
 import time
 import re
-
+import shutil
+import pdb
+import glob
 import pexpect
 
 # This is for local debugging!
@@ -47,11 +49,14 @@ class iRodsFileAdaptor(object):
         match = re.search("\$\{(.*)\}", self.localpath)
         if match:
             env_var = match.group(1)
-            logger.debug("Found: " + env_var)
+            logger.debug("Found: " + env_var + " in URL.")
+            logger.debug("Env list: " + str(os.environ))
             if os.environ.has_key(env_var):
-                self.localpath = re.sub(r'/\$\{.*\}/', os.environ[env_var], self.localpath)
+                self.localpath = re.sub(r'\$\{.*\}', os.environ[env_var], self.localpath)
+                #self.localpath = os.environ[env_var]
                 logger.debug("Expanding URL Path to: " + self.localpath)
                 return True
+        logger.debug("No expansion in: " + self.localpath)
         return False
     
     
@@ -88,6 +93,7 @@ class iRodsFileAdaptor(object):
 
                  
     def put_du(self, du):
+        start = time.time()
         logger.debug("Copy DU to iRod")
         du_items = du.list()
         for i in du_items.keys():     
@@ -98,23 +104,63 @@ class iRodsFileAdaptor(object):
                 self._put_file(local_filename, remote_path)
             except:
                 logger.debug("Could not copy: " + str(i))
+        logger.debug("Finished Put DU in: " + str(time.time()-start) + " sec.")
                 
     
     def get_du(self, du, target_url):
         #du_id = "du-7370d7b5-ed0b-11e1-95df-705681b3df0f"
+        start = time.time()
         du_id = du.id
         logger.debug("Get DU: " + str(du_id))
         if self.is_local:
             command = "cp -r %s %s"%(os.path.join(self.localpath, du_id), target_url)
+            source_path = os.path.join(self.localpath, du_id, "*")
+            target_path = target_url
+            logger.debug("Target and source host are localhost. Processing: %s" %(source_path))
+            expanded_path = glob.glob(source_path)
+            logger.debug("Expanded path: " + str(expanded_path))
+            for path in expanded_path:
+                if os.path.isdir(path):
+                    logger.debug("Source path %s is directory"%path)
+                    files = os.listdir(path)
+                    for i in files:
+                        try:
+                            os.symlink(os.path.join(files, i), target_path)
+                            os.chmod(os.path.join(target_path, os.path.basename(path)), 0777)
+                        except:
+                            self.__print_traceback()
+                else:
+                    try:
+                        os.symlink(path, os.path.join(target_path, os.path.basename(path)))
+                    except:
+                        self.__print_traceback()
+
         else:
-            command = "iget -r %s %s"%(du_id, target_url)
-        logger.debug(command)
-        self.__run_command(command)
-        if target_url==".":
-            target_url = os.getcwd()
-        command = "mv %s/* %s"%(os.path.join(target_url, du_id), target_url)
-        self.__run_command(command)
-        logger.debug("Finished Get DU")
+            command = "iget -f -r %s %s"%(du_id, target_url)
+            logger.debug(command)
+            self.__run_command(command)
+            
+            full_path = os.path.join(target_url, du_id)
+            #logger.debug("Path: " + str(full_path) + " Exists: " + str(os.path.exists(full_path)))
+            #while os.path.exists(full_path)==False:
+            #    time.sleep(1)
+    
+            for i in os.listdir(full_path):
+                try:
+                    logger.debug("chmod " + str(i))
+                    os.chmod(os.path.join(full_path, i), 0777)
+                    logger.debug("move " + str(i))
+                    shutil.move(os.path.join(full_path, i), target_url)
+                except:
+                    self.__print_traceback()
+    
+            shutil.rmtree(full_path, ignore_errors=True)
+            #time.sleep(2)
+            #if target_url==".":
+            #    target_url = os.getcwd()
+            #command = "mv %s/* %s"%(os.path.join(target_url, du_id), target_url)
+            #self.__run_command(command)
+            logger.debug("Finished Get DU " + du.id + " in: " + str(time.time()-start) + " sec.")
         
    
     def copy_du(self, du, pd_new):
@@ -136,15 +182,25 @@ class iRodsFileAdaptor(object):
     # Pure File Management APIs
     def _put_file(self, source, target):
         logger.debug("Put file: %s to %s"%(source, target))
+        start = time.time()
         if self.is_local:
-            command = "cp -r %s %s"%(os.path.join(source, target))
+            command = "cp -r %s %s"%(source, target)
         else:
             command = "iput -f -R %s %s %s"%(self.resource_group, source, target)
         self.__run_command(command)
-        home_directory= self.__run_command("ipwd")[0].strip()
-        full_filename = os.path.join(home_directory, target)
-        command = "irepl-osg -f %s -G %s"%(full_filename, self.resource_group)
-        self.__run_command(command) 
+        put_time = time.time() - start
+        number_replica = 0
+        if self.is_local==False:
+            #pdb.set_trace()
+            home_directory= self.__run_command("ipwd")[0].strip()
+            full_filename = os.path.join(home_directory, target)
+            command = "irepl-osg -f %s -G %s"%(full_filename, self.resource_group)
+            output = self.__run_command(command) 
+            for i in output:
+                if i.find("copied") > 0 or i.find("replica")>0:
+                    number_replica = number_replica + 1 
+        rep_time = time.time() - start - put_time
+        logger.info("Upload;Replication;Total;File Size;Backend;Number Replica: %f;%f;%f;%d;%s;%d"%(put_time, rep_time, time.time()-start, os.path.getsize(source), self.resource_group, number_replica))
          
 
     def transfer(self, source_url, target_url):
@@ -167,14 +223,14 @@ class iRodsFileAdaptor(object):
     def __print_traceback(self):
         exc_type, exc_value, exc_traceback = sys.exc_info()
         print "*** print_tb:"
-        traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
+        traceback.print_tb(exc_traceback, limit=1, file=sys.stderr)
         print "*** print_exception:"
         traceback.print_exception(exc_type, exc_value, exc_traceback,
-                              limit=2, file=sys.stdout)
+                              limit=2, file=sys.stderr)
     
     
 def test_irods():
-    irods = iRodsFileAdaptor("irods://gw68/${OSG_DATA}/?vo=osg&resource-group=osgGridFtpGroup")
+    irods = iRodsFileAdaptor("irods://gw68/${OSG_DATA}/osg/home/luckow/?vo=osg&resource-group=osgGridFtpGroup")
     irods.initialize_pilotdata()
     irods.create_du("du-7370d7b5-ed0b-11e1-95df-705681b3df0f")
     irods._put_file("test.txt", "du-7370d7b5-ed0b-11e1-95df-705681b3df0f/test.txt")
