@@ -41,7 +41,7 @@ except:
 try:
     from pilot.filemanagement.irods_adaptor import iRodsFileAdaptor
 except:
-    logger.warn("Goggle Storage package not found.") 
+    logger.warn("iRods Storage package not found.") 
 
 
 
@@ -237,13 +237,15 @@ class PilotData(PilotData):
             # initialize file adaptor
             if self.service_url.startswith("ssh:"):
                 logger.debug("Use SSH backend")
-                self.__filemanager = SSHFileAdaptor(self.service_url)
+                self.__filemanager = SSHFileAdaptor(self.service_url,
+                                                    self.security_context, 
+                                                    self.pilot_data_description)
             elif self.service_url.startswith("http:"):
                 logger.debug("Use WebHDFS backend")
                 self.__filemanager = WebHDFSFileAdaptor(self.service_url)
             elif self.service_url.startswith("go:"):
                 logger.debug("Use Globus Online backend")
-                self.__filemanager = GSFileAdaptor(self.service_url)
+                self.__filemanager = GlobusOnlineFileAdaptor(self.service_url)
             elif self.service_url.startswith("gs:"):
                 logger.debug("Use Google Cloud Storage backend")
                 self.__filemanager = GSFileAdaptor(self.service_url, self.security_context)
@@ -464,7 +466,10 @@ class DataUnit(DataUnit):
             self.data_unit_description = data_unit_description        
             self.pilot_data=[]
             self.state = State.New
-            self.data_unit_items = DataUnitItem.create_data_unit_list(self, self.data_unit_description["file_urls"]) 
+            self.data_unit_items=[]
+            if self.data_unit_description.has_key("file_urls"):
+                self.data_unit_items = DataUnitItem.create_data_unit_list(self, self.data_unit_description["file_urls"]) 
+
             self.url = None
 
             # register a data unit as top-level entry in Redis
@@ -539,7 +544,10 @@ class DataUnit(DataUnit):
    
     
     def get_state(self):
-        """ Return current state of DataUnit """        
+        """ Return current state of DataUnit """
+        # update remote state
+        du_dict = CoordinationAdaptor.get_du(self.url)
+        self.state = du_dict["state"]
         return self.state  
     
     
@@ -553,8 +561,8 @@ class DataUnit(DataUnit):
             i.join()
         
         # Wait for state to change
-        while self.state!=State.Running and self.state!=State.Failed:
-            logger.debug("State: %s"%self.state)
+        while self.state!=State.Running and self.get_state()!=State.Failed:
+            logger.debug("Waiting DU State: %s"%self.state)
             time.sleep(2)
     
     
@@ -569,6 +577,7 @@ class DataUnit(DataUnit):
     
     def get_pilot_data(self):
         """ get a list of pilot data that have a copy of this PD """
+        self.__restore_state()
         return self.pilot_data
     
     
@@ -602,6 +611,7 @@ class DataUnit(DataUnit):
     def _update_state(self, state):
         """ Internal method for updating state"""
         self.state=state
+        logger.debug("Update DU state to " + state + " Number of PD: " + str(len(self.pilot_data)))
         if len(self.pilot_data) > 0: 
             CoordinationAdaptor.update_du(self)
 
@@ -806,4 +816,4 @@ def test_data_unit_add_file():
 if __name__ == "__main__":
     #test_data_unit_add_file()
     test_du_reconnect()
-    
\ No newline at end of file
+    
